// DemoInterface.cpp: implementation of the CDemoInterface class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CommTest.h"
#include "DemoInterface.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

int CDemoInterface::m_iRun  = 0;
int CDemoInterface::m_iHold = 0;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDemoInterface::CDemoInterface()
{
   m_iInterfaceType = MODE_DEMO;
}

CDemoInterface::~CDemoInterface()
{
}

/////////////////////////////////////////////////////////////////////////////
// Function name: CDemoInterface::SendCommand
//
// Description:   Sends and validates a command string to the instrument
/////////////////////////////////////////////////////////////////////////////
BOOL CDemoInterface::SendCommand(CString strCmd)
{
   // Always return TRUE... we're fake, nothing can go wrong
   m_strCmd = strCmd;
   return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// Function name: CDemoInterface::ReadResponse
//
// Description:   Reads from the IO port until it reaches the end of the data
//                or runs out of buffer size
/////////////////////////////////////////////////////////////////////////////
int CDemoInterface::ReadResponse(CString &strBuff)
{
   static float temp = 30.0f;

   if (m_strCmd.Find("?") != -1) { 
      // Make sure it's a question
      if (m_strCmd.Find("AUXE") != -1) {
         strBuff = "0";
      }
      else if (m_strCmd.Find("CCHR") != -1) {
         strBuff = "C";
      }
      else if (m_strCmd.Find("CCNF") != -1) {
         strBuff = "2";
      }
      else if (m_strCmd.Find("DEVN") != -1) {
         strBuff = "1.0";
      }
      else if (m_strCmd.Find("FVAL") != -1) {
         strBuff = "30.0";
      }
      else if (m_strCmd.Find("IDEN") != -1) {
         strBuff = "DEMO CONTROLLER";
      }
      else if (m_strCmd.Find("INTN") != -1) {
         strBuff = "1";
      }
      else if (m_strCmd.Find("ITIM") != -1) {
         strBuff = "1:00:00";
      }
      else if (m_strCmd.Find("IVAL") != -1) {
         strBuff = "30.0";
      }
      else if (m_strCmd.Find("LLFT") != -1) {
         strBuff = "50";
      }
      else if (m_strCmd.Find("NUML") != -1) {
         strBuff = "100";
      }
      else if (m_strCmd.Find("NXTI") != -1) {
         strBuff = "2";
      }
      else if (m_strCmd.Find("PRMG") != -1) {
         strBuff = "1";
      }
      else if (m_strCmd.Find("PROG") != -1) {
         strBuff = "PROGRAM";
      }
      else if (m_strCmd.Find("PVAR") != -1) {
         temp += ((float)(rand() % 11) - 5.0f) / 100.0f;
         strBuff.Format("%.1f", temp);
      }
      else if (m_strCmd.Find("REFG1") != -1) {
         strBuff = "26.2,32.9,14.4,28.0,32.0,40.3,19.3,23.8,2";
      }
      else if (m_strCmd.Find("REFG") != -1) {
         strBuff = "0,0,0,0,0,0,0,0,0";
      }
      else if (m_strCmd.Find("SETP") != -1) {
         strBuff = "30.0";
      }
      else if (m_strCmd.Find("STAT") != -1) {
         strBuff.Format("%d", m_iRun | m_iHold);
      }
      else if (m_strCmd.Find("SCOD") != -1) {
         if (m_iRun) {
            strBuff = "1";
         }
         else {
            strBuff = "5";
         }
      }
      else if (m_strCmd.Find("THTL") != -1) {
         if (m_iRun) {
            float thtl = (30.0f - temp)*10.0f;
            if (thtl > 100) {
               thtl = 100;
            }
            strBuff.Format("%.0f", thtl);
         }
         else {
            strBuff = "0";
         }
      }
      else if (m_strCmd.Find("TLFT") != -1) {
         strBuff = "0:43:28";
      }
      else if (m_strCmd.Find("CONF") != -1) {
         strBuff = "3";
      }
      else {
         strBuff = "0";
      }
   }
   else if (m_strCmd.GetLength() > 0) {
      // Not a query, return 0
      strBuff = "0";

      if (m_strCmd.Find("HOLD") != -1) {
         if (m_iRun) {
            m_iHold = 2;
         }
      }
      else if (m_strCmd.Find("RUNP") != -1 || m_strCmd.Find("RESM") != -1) {
         m_iRun = 1;
         m_iHold = 0;
      }
      else if (m_strCmd.Find("STOP") != -1) {
         m_iRun = 0;
         m_iHold = 0;
      }
   }
   else {
      strBuff = "";
   }
   m_strCmd = "";

   return strBuff.GetLength();
}

BOOL CDemoInterface::IsValidComm()
{
   // Always valid comm in demo mode
   return TRUE;
}

void CDemoInterface::InvalidateComm()
{
   // Doesn't do anything in Demo mode
   return;
}
