// GPIBInterface.cpp: implementation of the CGPIBInterface class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CommTest.h"
#include "GPIBInterface.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGPIBInterface::CGPIBInterface(int address)
{
   // Lock this object
   m_CommCritSect.Lock();

   int i = 0;
   // Look for a board...
   while ((m_iCom = ibdev(i, address, 0, T1s, 1, 0)) < 0 && i < 16) {

      // Haven't found one yet...
      i++;
   }

   //TRACE("COMM: %d\n", m_iCom);

   m_iInterfaceType = MODE_GPIB;
}

CGPIBInterface::~CGPIBInterface()
{
   if (m_iCom != -1) {
      // Take the board off line if we have a valid handle
      ibonl(m_iCom, 0);
   }

   // Unlock this object
   m_CommCritSect.Unlock();
}

/////////////////////////////////////////////////////////////////////////////
// Function name: CGPIBInterface::SendCommand
//
// Description:   Sends and validates a command string to the instrument
/////////////////////////////////////////////////////////////////////////////
BOOL CGPIBInterface::SendCommand(CString strCmd)
{

   //Sleep(25);

   if (m_iCom != -1) 
   {
      // Write the buffer out to the com port
      ibwrt(m_iCom, strCmd.GetBuffer(0), strCmd.GetLength());

      TRACE("strCmd: %s\n", strCmd);
      //TRACE("sent: %d\n", ibcntl);

      if (ibcntl == strCmd.GetLength()) {
         // Ok, we sent out all of the data
         return TRUE;
      }
   }

   // Something went wrong, return false.
   return FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// Function name: CGPIBInterface::ReadResponse
//
// Description:   Reads from the IO port until it reaches the end of the data
//                or runs out of buffer size
/////////////////////////////////////////////////////////////////////////////
int CGPIBInterface::ReadResponse(CString &strBuff)
{
   ULONG numin = 0;
   char szBuff[256];    // Seems reasonable

   if (m_iCom != -1) {
      // Read from the port until the GPIB driver decides we're done (EOI?)
      ibrd(m_iCom, szBuff, 255);
      numin = ibcntl;
   }

   // If the last thing we read was a carriage return, we want to throw it away
	while(numin > 0 && (szBuff[numin-1] == '\r' || szBuff[numin-1] == '\n'))
	{
      numin--;
   }

   // NULL terminate this string
   szBuff[numin] = '\0';
   strBuff = szBuff;
   // return strBuff.GetLength();

	TRACE("recv: %s\n", strBuff);

	return numin;
}

BOOL CGPIBInterface::IsValidComm()
{
   // Tells whether or not we have a valid comm object
   return (m_iCom != -1);
}

void CGPIBInterface::InvalidateComm()
{
   if (m_iCom != -1) {
      // Take the board off line if we have a valid handle
      ibonl(m_iCom, 0);
      m_iCom = -1;
   }
}
