// TCPInterface.cpp: implementation of the CTCPInterface class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CommTest.h"
#include "TCPInterface.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTCPInterface::CTCPInterface(CString address, int port)
{
   m_CommCritSect.Lock();

   // I guess x.x.x.x is the shortest a valid address could be, so check for it.
   if (address.GetLength() >= 7) 
   {
      SOCKADDR_IN addr;
      addr.sin_family = AF_INET;
      addr.sin_port   = htons(port);    // The default port of the board we're talking to
      *(ULONG *)&(addr.sin_addr)   = inet_addr(address.GetBuffer(0));

      TRACE("address: %s\n", address);

      m_hSocket = socket(AF_INET, SOCK_STREAM, 0);
      if (m_hSocket != INVALID_SOCKET) 
      {
         // Set the timeout for the socket
         int timeout = 1000;
         setsockopt(m_hSocket, SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout, sizeof(timeout));
         setsockopt(m_hSocket, SOL_SOCKET, SO_SNDTIMEO, (char *)&timeout, sizeof(timeout));
         if (connect(m_hSocket, (sockaddr *)&addr, sizeof(addr)) == SOCKET_ERROR) 
         {
            // The connection failed, invalidate the socket
            closesocket(m_hSocket);
            m_hSocket = INVALID_SOCKET;
         }
      }
      else
      {
         TRACE("error: %d\n", WSAGetLastError());
      }
   }
   else {
      // Looks like an invalid address
      m_hSocket = INVALID_SOCKET;
   }

   if(m_hSocket != INVALID_SOCKET)
   {
      TRACE("CONNECTED!\n");
   }

   m_iInterfaceType = MODE_TCPIP;
}

CTCPInterface::~CTCPInterface()
{
   if (IsValidComm()) {
      closesocket(m_hSocket);
      m_hSocket = INVALID_SOCKET;
   }

   TRACE("Disconnected!\n");  

   m_CommCritSect.Unlock();
}

/////////////////////////////////////////////////////////////////////////////
// Function name: CTCPInterface::SendCommand
//
// Description:   Sends and validates a command string to the instrument
/////////////////////////////////////////////////////////////////////////////
BOOL CTCPInterface::SendCommand(CString strCmd)
{
   ULONG numout;

   if (IsValidComm()) {
      // We're talking via TCP/IP, write the buffer out to the socket
      //numout = send(m_hSocket, strCmd.GetBuffer(0), strCmd.GetLength(), 0);
   
      numout = send(m_hSocket, strCmd.GetBuffer(0), strCmd.GetLength(), 0);

      TRACE("numout: %d\n", numout);

      if (numout == (ULONG)strCmd.GetLength()) {
         // Ok, we sent out all of the data
         return TRUE;
      } 
   }

   // Something went wrong, return false.

   return FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// Function name: CTCPInterface::ReadResponse
//
// Description:   Reads from the IO port until it reaches the end of the data
//                or runs out of buffer size
/////////////////////////////////////////////////////////////////////////////
int CTCPInterface::ReadResponse(CString &strBuff)
{
   ULONG numin;
   char recvbuff[256];

   strBuff = "";

   if (IsValidComm()) {
      // We're communicating via TCP/IP

      // Read from the socket until we hit the end of the data or the ReadFile  operation fails
      BOOL GotResponse = FALSE;
      while (!GotResponse) {
         // Try reading, oh, let's say 256 bytes at a time.  It will return before
         // it gets all 256 on shorter reads.
         numin = recv(m_hSocket, recvbuff, 256, 0);
         if (numin != 0 && numin != SOCKET_ERROR) {
            UINT  idx = 0;

            while (idx < numin && recvbuff[idx] != '\r' && recvbuff[idx] != '\n') {
               // Throw away plain linefeeds
               if (recvbuff[idx] != '\n') {
                  // Add a character to the buffer (We're using \r as our terminating character).
                  strBuff += recvbuff[idx];
               }

               idx++;
            }
            
            if (idx < numin) {
               // Time to exit the loop
               GotResponse = TRUE;
            }
         }
         else {
            // Something went wrong, get out of here.  Clear out anything we
            // may have already received.
            GotResponse = TRUE;
            strBuff = "";

				TRACE("error: %d\n", WSAGetLastError());
         }
      }
   }

   TRACE("strBuff: %s\n", strBuff);

   return strBuff.GetLength();
}

///////////////////////////////////////////////////////////////////
//    Name: IsValidComm
//
//    Desc: checks if we have a valid connection or not
//////////////////////////////////////////////////////////////////
BOOL CTCPInterface::IsValidComm()
{
   BOOL retval = FALSE;
   if (m_hSocket != INVALID_SOCKET) {
      // We still have a valid socket
      retval = TRUE;
   }

   return retval;
}

///////////////////////////////////////////////////////////////////
//    Name: InvalidateComm
//
//    Desc: pulls the plug on our connection
//////////////////////////////////////////////////////////////////
void CTCPInterface::InvalidateComm()
{
   // Disconnect the call, which will invalidate the comm handle
   if (IsValidComm()) {
      // Pull the plug on the connection
      closesocket(m_hSocket);
      m_hSocket = INVALID_SOCKET;
   }
}
